local addonName, ns = ...

local module = {}
module.name = "CombatText"
module.Init = function()
	if not SCDB.modules[module.name] then return end
	if SCDB[module.name] == nil then SCDB[module.name] = {} end
	if SCDB[module.name]["StartX"] == nil then SCDB[module.name]["StartX"] = 305 end
	if SCDB[module.name]["EndX"] == nil then SCDB[module.name]["EndX"] = 305 end
	if SCDB[module.name]["StartY"] == nil then SCDB[module.name]["StartY"] = 550 end
	if SCDB[module.name]["EndY"] == nil then SCDB[module.name]["EndY"] = 300 end
	if SCDB[module.name]["Raid announce"] == nil then SCDB[module.name]["Raid announce"] = false end
	if SCDB[module.name]["Party announce"] == nil then SCDB[module.name]["Party announce"] = false end
	if SCDB[module.name]["Yell announce"] == nil then SCDB[module.name]["Yell announce"] = true end
	if SCDB[module.name]["Say announce"] == nil then SCDB[module.name]["Say announce"] = false end
	if SCDB[module.name]["Play sounds"] == nil then SCDB[module.name]["Play sounds"] = true end
	local opts = SCDB[module.name]

	local lastKillTime = 0
	local killingStreak = 0
	local multiKill = 0
	local MULTI_KILL_HOLD_TIME = 11.5
	local bit_band = bit.band
	local bit_bor = bit.bor
	local pendingSound

	local spreeSounds = {
		[3] = "Killing_Spree",
		[4] = "Dominating",
		[5] = "Mega_Kill",
		[6] = "Unstoppable",
		[7] = "Wicked_Sick",
		[8] = "Monster_Kill",
		[9] = "Ludicrous_Kill",
		[10] = "God_Like",
		[11] = "Holy_Shit"
	}

	local multiSounds = {
		[2] = "Double_Kill",
		[3] = "Triple_Kill",
	}
	
	local path = "Interface\\AddOns\\" .. addonName .. "\\media\\sounds\\%s.mp3"
	
	local function hasFlag(flags, flag)
		return bit_band(flags, flag) == flag
	end

	-- by Evl
	local playSounds = function()
		local multiFileName = multiSounds[math.min(3, multiKill)]
		local spreeFileName = spreeSounds[math.min(11, killingStreak)]

		if multiFileName then
			PlaySoundFile(string.format(path, multiFileName))
		end

		if spreeFileName then
			local spreeFilePath = string.format(path, spreeFileName)

			if not multiFileName then
				PlaySoundFile(spreeFilePath)
			else
				pendingSound = spreeFilePath
			end
		end
	end

	local extraCombatText = CreateFrame("Frame")
	extraCombatText:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	extraCombatText:RegisterEvent("PLAYER_DEAD")
	extraCombatText:RegisterEvent("ADDON_LOADED")
	extraCombatText:SetScript("OnEvent", function(self, event, ...)
		if event == "COMBAT_LOG_EVENT_UNFILTERED" then
			timestamp, eventType, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags = ...
			-- Killing blows
			if eventType == "PARTY_KILL" and hasFlag(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) and hasFlag(destFlags, COMBATLOG_OBJECT_TYPE_PLAYER) then
				if COMBAT_TEXT_TYPE_INFO then
					if SCDB[module.name]["Raid announce"] then
						SendChatMessage("Killing Blow! (" .. destName .. ")", "RAID")
					end
					if SCDB[module.name]["Party announce"] then
						SendChatMessage("Killing Blow! (" .. destName .. ")", "PARTY")
					end
					if SCDB[module.name]["Yell announce"] then
						SendChatMessage("Killing Blow! (" .. destName .. ")", "YELL")
					end
					if SCDB[module.name]["Say announce"] then
						SendChatMessage("Killing Blow! (" .. destName .. ")", "SAY")
					end
				end

				local now = GetTime()
		
				if lastKillTime + MULTI_KILL_HOLD_TIME > now then
					multiKill = multiKill + 1
				else
					multiKill = 1
				end
		
				lastKillTime = now
				killingStreak = killingStreak + 1
				if opts["Play sounds"] then
					playSounds()
				end
				
			-- Interrupts
			elseif eventType == "SPELL_INTERRUPT" and hasFlag(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) then
				local extraSpellID, extraSpellName, extraSpellSchool = select(12, ...)
				if SCDB[module.name]["Raid announce"] then
					SendChatMessage(format("%s Interrupted!", GetSpellLink(extraSpellID)), "RAID")
				end
				if SCDB[module.name]["Party announce"] then
					SendChatMessage(format("%s Interrupted!", GetSpellLink(extraSpellID)), "PARTY")
				end
				if SCDB[module.name]["Yell announce"] then
					SendChatMessage(format("%s Interrupted!", GetSpellLink(extraSpellID)), "YELL")
				end
				if SCDB[module.name]["Say announce"] then
					SendChatMessage(format("%s Interrupted!", GetSpellLink(extraSpellID)), "SAY")
				end
			elseif eventType == "SPELL_AURA_APPLIED" and destGUID == UnitGUID("target") then
--[[				local spellID = select(9, ...)
				if spellID == 871 then
					CombatText_AddMessage("SHIELD WALL UP!", COMBAT_TEXT_SCROLL_FUNCTION, .7, .7, 1, "crit", nil)
					PlaySoundFile("Interface\\AddOns\\evl_CombatText\\sounds\\sohelp02.mp3")
				end]]
			end
		elseif event == "PLAYER_DEAD" then
			killingStreak = 0
		end
	end)
	
	extraCombatText:SetScript("OnUpdate", function(self, elapsed)
		if self.lastUpdate == nil then self.lastUpdate = 0 end
		self.lastUpdate = self.lastUpdate + elapsed
		if self.lastUpdate > 2 then
			self.lastUpdate = 0
			if pendingSound then
				PlaySoundFile(pendingSound)
				pendingSound = nil
			end
		end
	end)
	
	COMBAT_TEXT_TYPE_INFO["ENTERING_COMBAT"] = { r = .1, g = .1, b = 1 }
	COMBAT_TEXT_TYPE_INFO["LEAVING_COMBAT"] = { r = .1, g = .1, b = 1 }
	COMBAT_TEXT_TYPE_INFO["COMBO_POINTS"] = { r = .7, g = .7, b = 1 }
	
	COMBAT_TEXT_DEFLECT = "Deflect"
	COMBAT_TEXT_REFLECT = "Reflect"
	COMBAT_TEXT_IMMUNE = "Immune"
	COMBAT_TEXT_RESIST = "Resist"
	COMBAT_TEXT_ABSORB = "Absorb"
	COMBAT_TEXT_BLOCK = "Block"
	COMBAT_TEXT_DODGE = "Dodge"
	COMBAT_TEXT_PARRY = "Parry"
	COMBAT_TEXT_EVADE = "Evade"
	COMBAT_TEXT_MISS = "Miss"

	DEFLECT = "Deflect"
	REFLECT = "Reflect"
	IMMUNE = "Immune"
	RESIST = "Resist"
	ABSORB = "Absorb"
	BLOCK = "Block"
	DODGE = "Dodge"
	PARRY = "Parry"
	EVADE = "Evade"
	MISS = "Miss"

	BLOCK_TRAILER = "(Block %d)"
	ABSORB_TRAILER = "(Absorb %d)"
	RESIST_TRAILER = "(Resist %d)"

	ENTERING_COMBAT = "+COMBAT+"
	LEAVING_COMBAT = "-COMBAT-"
	COMBAT_TEXT_SCROLLSPEED = 2
	
	HEALTH_LOW = "Low Health"
	
	hooksecurefunc("CombatText_UpdateDisplayedMessages", function ()
		COMBAT_TEXT_LOCATIONS = {
			startX  = opts.StartX,
			startY = opts.StartY * COMBAT_TEXT_Y_SCALE,
			endX = opts.EndX,
			endY = opts.EndY * COMBAT_TEXT_Y_SCALE
		}
	end)

end
tinsert(SuperClassic.modules, module) -- finish him!