local module = {}
module.name = "Unit Frames"
module.Init = function()
    if not SCDB.modules[module.name] then return end
    if SCDB[module.name] == nil then SCDB[module.name] = {} end
    if SCDB[module.name]["3D portaits"] == nil then SCDB[module.name]["3D portaits"] = true end
    if SCDB[module.name]["Debuffs On Top"] == nil then SCDB[module.name]["Debuffs On Top"] = true end
    
    local config = {
        largeAuraSize = 32,
        smallAuraSize = 24,
        pos = {
            [1] = { a = "CENTER",     x = -150, y = -250 }, -- PlayerFrame
            [2] = { a = "CENTER",     x =  150, y = -250 }, -- TargetFrame
            [3] = { a = "CENTER",     x = -440, y =   40 }, -- PartyFrame
        },
    }
    
    hooksecurefunc(getmetatable(PlayerFrameHealthBar).__index, "Show", function(self, ...)
        if self:GetParent().unit then
            if self.styled == nil then
                self:SetStatusBarTexture("Interface\\TokenFrame\\UI-TokenFrame-CategoryButton")
                self:GetStatusBarTexture():SetDesaturated(1)
                self:GetStatusBarTexture():SetTexCoord(0, 1, 0.609375, 0.796875)
                self:GetStatusBarTexture():SetHorizTile(true)
                self.bg = self:CreateTexture(self:GetName().."Background", "BACKGROUND")
                self.bg:SetTexture(0,0,0,.8)
                self.bg:SetAllPoints(self)
                self.styled = true
            end
        end
    end)
    
    local cc = CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[select(2, UnitClass("player"))] or RAID_CLASS_COLORS[select(2, UnitClass("player"))] or  { r=.5, g=.5, b=.5 }

    local function PaintTextures()
        for i,v in pairs({
            MainMenuBarTexture0,
            MainMenuBarTexture1,
            MainMenuBarTexture2,
            MainMenuBarTexture3,
            MainMenuBarTexture4,
            MainMenuBarLeftEndCap,
            MainMenuBarRightEndCap,
            MainMenuMaxLevelBar0,
            MainMenuMaxLevelBar1,
            PlayerFrameTexture,
            TargetFrameTextureFrameTexture,
            PetFrameTextureFrameTexture,
            FocusFrameTextureFrameTexture,
            TargetFrameToTTextureFrameTexture,
            FocusFrameToTTextureFrameTexture,
            MinimapBorder,
            select(1, TimeManagerClockButton:GetRegions()),
        }) do
            if v:GetObjectType() == "Texture" then
                v:SetDesaturated(1)
                if SCDB["Main"].ClassColored == true then
                    v:SetVertexColor(cc.r*.5, cc.g*.5, cc.b*.5)
                else
                    v:SetVertexColor(.5,.5,.5)
                end
            end
        end
    end

    local ef=CreateFrame("frame")
    ef:RegisterEvent("UNIT_AURA")
    ef:SetScript("OnEvent", function(object, event, unit, ...)
        if object.unit ~= unit  then return end
        if not UnitCanAssist("player", unit) then return nil end
        local d, c, t
        
        PaintTextures()
        
        for i=1,40 do
            d = select(5,UnitAura(unit, i, "HARMFUL"))
            if d then
                c=DebuffTypeColor[d]
                
                if (unit == "player") then
                    t=PlayerFrameTexture
                elseif (unit == "target") then
                    t=TargetFrameTextureFrameTexture
                elseif (unit == "targettarget") then
                    t=TargetFrameToTTextureFrameTexture
                elseif (unit == "focus") then
                    t=FocusFrameTextureFrameTexture
                elseif (unit == "focustarget") then
                    t=FocusFrameToTTextureFrameTexture
                end
                t:SetVertexColor(c.r, c.g, c.b)
                break
                
            end
        end
    end)
    
    local allframes = {
        PlayerFrame,
        TargetFrame,
        TargetFrameToT,
        PartyMemberFrame1,
        ArenaEnemyFrame1,
        PetFrame,
    }
   
    SCDB[module.name].opt = SCDB[module.name].opt or {}
    
    PlayerFrame:SetUserPlaced(false)
    
    for i,v in pairs(allframes) do
        SCDB[module.name].opt[v:GetName()] = SCDB[module.name].opt[v:GetName()] or {}
                
        v.default = v:GetWidth()
        
        v.drag = CreateFrame("frame", v:GetName().."Drag", UIParent)
        
        if SCDB[module.name].opt[v:GetName()].w ~= nil then
            v.drag:SetWidth(SCDB[module.name].opt[v:GetName()].w)
            v.drag:SetHeight(SCDB[module.name].opt[v:GetName()].h)
        else
            v.drag:SetWidth(v:GetWidth())
            v.drag:SetHeight(v:GetHeight())
        end
        
        v:RegisterForDrag()
        
        if SCDB[module.name].opt[v:GetName()].x == nil then
            local point, relativeTo, relativePoint, xOfs, yOfs = v:GetPoint(v:GetNumPoints())
            v.drag:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs)
        else
            v.drag:SetPoint("CENTER", UIParent, "BOTTOMLEFT", SCDB[module.name].opt[v:GetName()].x, SCDB[module.name].opt[v:GetName()].y)
        end
        
        v.drag:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", })
        v.drag:SetBackdropColor(1, 1, 1, .4)
        
        v.drag:SetMovable(true)
        v.drag:SetResizable(true)
        v.drag:EnableMouse(false)
        v.drag:SetClampedToScreen(true)
        v.drag:RegisterForDrag("LeftButton")
        v.drag:SetScript("OnDragStart", function(self) 
            self:StartMoving()
        end)
        v.drag:SetScript("OnDragStop", function(self) 
            self:StopMovingOrSizing()
            local x, y = self:GetCenter()
            if SCDB[module.name].opt[v:GetName()] == nil then SCDB[module.name].opt[v:GetName()] = {} end
            SCDB[module.name].opt[v:GetName()].x = x
            SCDB[module.name].opt[v:GetName()].y = y
        end)
        v.drag:Hide()
        v.drag.txt = v.drag:CreateFontString(v.drag:GetName().."DragText", "OVERLAY", "GameFontHighlight")
        v.drag.txt:SetPoint("CENTER")
        v.drag.txt:SetText(v:GetName())
        
        v.drag.resize = CreateFrame("button", v.drag:GetName().."ResizeButton", v.drag)
        v.drag:SetMinResize(16, 16)
        v.drag:SetMaxResize(600, 400)
        v.drag.resize:SetPoint("BOTTOMRIGHT")
        v.drag.resize:SetWidth(16)
        v.drag.resize:SetHeight(16)
        v.drag.resize:SetNormalTexture("Interface\\CHATFRAME\\UI-ChatIM-SizeGrabber-Up")
        v.drag.resize:SetHighlightTexture("Interface\\CHATFRAME\\UI-ChatIM-SizeGrabber-Highlight")
        v.drag.resize:SetPushedTexture("Interface\\CHATFRAME\\UI-ChatIM-SizeGrabber-Down")
        v.drag.resize:SetScript("OnMouseDown", function(self) v.drag:StartSizing() end)
        v.drag.resize:SetScript("OnMouseUp", function(self)
            local parent = v.drag
            parent:StopMovingOrSizing()
            if v:GetName():find("1") then
                local tmp = v:GetName():sub(1, #v:GetName()-1)
                for k=1,4 do
                    _G[tmp..k]:SetScale(v.drag:GetWidth()/v.default)
                end
            else
                v:SetScale(v.drag:GetWidth()/v.default)
            end
            local w, h = parent:GetWidth(), parent:GetHeight()
            if SCDB[module.name].opt[v:GetName()] == nil then SCDB[module.name].opt[v:GetName()] = {} end
            SCDB[module.name].opt[v:GetName()].w = w
            SCDB[module.name].opt[v:GetName()].h = h
        end)
        
        v.drag.reset = CreateFrame("button", v:GetName().."ResetButton", v.drag, "UIPanelCloseButton")
        v.drag.reset:SetWidth(22)
        v.drag.reset:SetHeight(22)
        v.drag.reset:SetPoint("TOPRIGHT", 4, 4)
        v.drag.reset:SetScript("OnClick", function()
            if v:GetScale() ~= 1 then
                v:SetScale(1)
                local w, h = v:GetWidth(), v:GetHeight()
                v.drag:SetWidth(w)
                v.drag:SetHeight(h)
                if SCDB[module.name].opt[v:GetName()] == nil then SCDB[module.name].opt[v:GetName()] = {} end
                SCDB[module.name].opt[v:GetName()].w = w
                SCDB[module.name].opt[v:GetName()].h = h
                print("Size of |cff0077dd"..v:GetName().."|r was rezet to default")
            end
        end)
        
        v.locked = true
    end
    
    local exprep = CreateFrame("StatusBar", "exprep", PlayerFrame)
    exprep:SetStatusBarTexture("Interface\\WorldStateFrame\\WorldState-CaptureBar")
    exprep:GetStatusBarTexture():SetTexCoord(0.8203125, 1.0, 0.34375, 0.484375)
    exprep:SetStatusBarColor(0, .8, 0)

    ---------------------------------------------------
    -- PLAYER
    ---------------------------------------------------
    local function UpdatePlayerFrame()
        PaintTextures()
        if not UnitHasVehicleUI("player") then
            PlayerPVPIcon:SetSize(50,50)
            PlayerPVPIcon:SetPoint("TOPLEFT", 25, -25)
            PlayerName:SetWidth(0.01)
            PlayerFrameGroupIndicatorText:ClearAllPoints()
            PlayerFrameGroupIndicatorText:SetPoint("BOTTOMLEFT", PlayerFrame, "TOP", 0, -20)
            PlayerFrameGroupIndicatorLeft:Hide()
            PlayerFrameGroupIndicatorMiddle:Hide()
            PlayerFrameGroupIndicatorRight:Hide()
            PlayerFrameHealthBar:ClearAllPoints()
            PlayerFrameHealthBar:SetPoint("TOPLEFT", 106, -24)
            PlayerFrameHealthBar:SetHeight(18)
            PlayerFrameHealthBarText:ClearAllPoints()
            PlayerFrameHealthBarText:SetPoint("RIGHT", PlayerFrameHealthBar, 0, 0)
            PlayerFrameManaBar:ClearAllPoints()
            PlayerFrameManaBar:SetPoint("TOPLEFT", 106, -43)
            PlayerFrameManaBar:SetHeight(10)
            PlayerFrameManaBarText:ClearAllPoints()
            PlayerFrameManaBarText:SetPoint("RIGHT", PlayerFrameManaBar, 0, 0)
            PlayerFrameManaBarText:SetFont(PlayerFrameManaBarText:GetFont(), 9, "OUTLINE")
            
            PlayerFrameFlash:SetTexture("")
            PlayerFrameFlash.SetTexture = function() end
            
            PlayerFrameAlternateManaBarBorder:Hide()
            PlayerFrameAlternateManaBarBorder.Show = function() end
            PlayerFrameAlternateManaBarBackground:Hide()
            PlayerFrameAlternateManaBarBackground.Show = function() end
            PlayerFrameAlternateManaBar:ClearAllPoints()
            PlayerFrameAlternateManaBar:SetPoint("TOPLEFT", 106, -53)
            PlayerFrameAlternateManaBar:SetHeight(10)
            PlayerFrameAlternateManaBar:SetWidth(PlayerFrameManaBar:GetWidth())
            PlayerFrameAlternateManaBar.SetWidth = function() end
            PlayerFrameAlternateManaBarText:ClearAllPoints()
            PlayerFrameAlternateManaBarText:SetPoint("RIGHT", PlayerFrameAlternateManaBar, 0, 0)
            PlayerFrameAlternateManaBarText:SetFont(PlayerFrameAlternateManaBarText:GetFont(), 8, "OUTLINE")
            PlayerStatusTexture:Hide()
            PlayerStatusTexture.Show = function() end
            
            if not PlayerFrameAlternateManaBar:IsShown() then
                if MainMenuExpBar:IsShown() and not ReputationWatchBar:IsShown() then
                    exprep:Show()
                    hooksecurefunc("MainMenuExpBar_Update", function()
                        exprep:SetMinMaxValues(MainMenuExpBar:GetMinMaxValues())
                        exprep:SetValue(MainMenuExpBar:GetValue())
                    end)
                    exprep:SetStatusBarColor(MainMenuExpBar:GetStatusBarColor())
                    exprep:SetScript("OnEnter", function(self)
                        GameTooltip:ClearLines()
                        GameTooltip:SetOwner(self, ANCHOR_TOPLEFT)
                        GameTooltip:AddLine(MainMenuBarExpText:GetText())
                        GameTooltip:Show()
                    end)
                    exprep:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
                elseif ReputationWatchBar:IsShown() then
                    exprep:Show()
                    hooksecurefunc("ReputationWatchBar_Update", function()
                        exprep:SetMinMaxValues(ReputationWatchStatusBar:GetMinMaxValues())
                        exprep:SetValue(ReputationWatchStatusBar:GetValue())
                    end)
                    exprep:SetStatusBarColor(ReputationWatchStatusBar:GetStatusBarColor())
                    exprep:SetScript("OnEnter", function(self)
                        GameTooltip:ClearLines()
                        GameTooltip:SetOwner(self, ANCHOR_TOPLEFT)
                        GameTooltip:AddLine(ReputationWatchStatusBarText:GetText())
                        GameTooltip:Show()
                    end)
                    exprep:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
                else
                    if exprep:IsShown() then exprep:Hide() end
                end
                exprep:ClearAllPoints()
                exprep:SetHeight(10)
                exprep:SetWidth(PlayerFrameHealthBar:GetWidth())
                exprep.SetWidth = function() end
                exprep:SetPoint("TOPLEFT", 106, -53)
            else
               if exprep:IsShown() then exprep:Hide() end
            end

            ---------------------------------------------------
            -- RUNES/TOTEMS/COMBOPOINTS
            ---------------------------------------------------
            local class = select(2, UnitClass("player"))
            
            if class == "SHAMAN" then
                for i = 1, 4 do
                    local totem = _G["TotemFrameTotem"..i]
                    if not totem.bg then
                        select(2, totem:GetChildren()):Hide()
                        select(2, totem:GetChildren()).Show = function() end
                        _G["TotemFrameTotem"..i.."IconTexture"]:SetTexCoord(.1, .9, .1, .9)
                        _G["TotemFrameTotem"..i.."Background"]:Hide()
                        _G["TotemFrameTotem"..i.."Background"].Show = function () end
                        _G["TotemFrameTotem"..i.."Duration"]:Hide()
                        _G["TotemFrameTotem"..i.."Duration"].Show = function() end
                        totem:ClearAllPoints()
                        if i == 1 then
                            totem:SetPoint("LEFT", TotemFrame, "LEFT", 2, 0)
                        else
                            totem:SetPoint("LEFT", _G["TotemFrameTotem"..(i-1)], "RIGHT", 2, 0)
                        end
                        _G["TotemFrameTotem"..i.."Icon"]:SetWidth(totem:GetWidth()-4)
                        _G["TotemFrameTotem"..i.."Icon"]:SetHeight(totem:GetWidth()-4)
                        totem.bg = CreateFrame("Frame", nil, totem)
                        totem.bg:SetFrameLevel(_G["TotemFrameTotem"..i.."Icon"]:GetFrameLevel() + 1)
                        totem.bg:SetWidth(totem:GetWidth())
                        totem.bg:SetHeight(totem:GetWidth())
                        totem.bg:SetPoint("CENTER", totem, "CENTER")
                        totem.bgt = totem.bg:CreateTexture(nil, "ARTWORK")
                        totem.bgt:SetAllPoints(totem.bg)
                        
                        totem.bgt:SetTexture("Interface\\Buttons\\UI-TotemBar")
                        totem.bgt:SetTexCoord(1 / 128, 35 / 128, 207 / 256, 240 / 256)
                        totem.bgt:SetDesaturated(1)
                    end
                end
                
            elseif class == "DEATHKNIGHT" then
                RuneFrame:ClearAllPoints()
                RuneFrame:SetPoint("TOPLEFT", PlayerFrameManaBar, "BOTTOMLEFT", -14, -17)
                for i = 1, 6 do
                    local rune = _G["RuneButtonIndividual"..i]
                    if not rune.bg then
                        rune:SetWidth(24)
                        rune:SetHeight(24)
                        select(1, _G["RuneButtonIndividual"..i.."Border"]:GetRegions()):SetTexture("")
                        _G["RuneButtonIndividual"..i.."Rune"]:SetTexCoord(.2, .8, .2, .8)
                        _G["RuneButtonIndividual"..i.."Cooldown"]:SetWidth(rune:GetWidth()-4)
                        _G["RuneButtonIndividual"..i.."Cooldown"]:SetHeight(rune:GetWidth()-4)
                        _G["RuneButtonIndividual"..i.."Rune"]:SetWidth(rune:GetWidth()-4)
                        _G["RuneButtonIndividual"..i.."Rune"]:SetHeight(rune:GetWidth()-4)
                        rune:ClearAllPoints()
                        if i == 1 then
                            rune:SetPoint("LEFT", RuneFrame, "LEFT", 2, 0)
                        else
                            rune:SetPoint("LEFT", _G["RuneButtonIndividual"..(i-1)], "RIGHT", 0, 0)
                        end
                        rune.bg = CreateFrame("Frame", nil, rune)
                        rune.bg:SetFrameLevel(rune:GetFrameLevel() + 1)
                        rune.bg:SetWidth(rune:GetWidth())
                        rune.bg:SetHeight(rune:GetWidth())
                        rune.bg:SetPoint("CENTER")
                        rune.bgt = rune.bg:CreateTexture(nil, "ARTWORK")
                        rune.bgt:SetAllPoints(rune.bg)
                        rune.bgt:SetTexture("Interface\\Buttons\\UI-TotemBar")
                        rune.bgt:SetTexCoord(1 / 128, 35 / 128, 207 / 256, 240 / 256)
                        rune.bgt:SetDesaturated(1)
                    end
                end
            end         


        else
            PlayerFrameHealthBar:SetHeight(12)
            PlayerFrameManaBar:SetHeight(12)
        end
    end
    hooksecurefunc("ReputationWatchBar_Update", UpdatePlayerFrame)
    hooksecurefunc("PlayerFrame_UpdateArt", UpdatePlayerFrame)
    hooksecurefunc("PlayerFrame_SequenceFinished", UpdatePlayerFrame)
    PlayerFrame:HookScript("OnEvent", function(self, event)
        if event=="PLAYER_ENTERING_WORLD" then
            UpdatePlayerFrame()
            MoveAllFrames()
        end
    end)
    
    function MoveAllFrames()
        for i,v in pairs(allframes) do
            if v.moved == nil then
                v:ClearAllPoints()
                v:SetPoint("TOPLEFT", v.drag, 0, 0)
                v.SetPoint = function() end
                v.moved = true
            end
            
            if v.locked == false then
                if v:GetName():find("1") then
                    local tmp = v:GetName():sub(1, #v:GetName()-1)
                    for k=1,4 do
                        _G[tmp..k]:SetScale(v.drag:GetWidth()/v.default)
                    end
                else
                    v:SetScale(v.drag:GetWidth()/v.default)
                end
            end
        end
    end
    MoveAllFrames()
    
    ---------------------------------------------------
    -- TARGET
    ---------------------------------------------------
    TargetFrame.nameBackground:Hide()
    TargetFrame:SetHitRectInsets(0,0,0,0)
    TargetFrame.deadText:ClearAllPoints()
    TargetFrame.deadText:SetPoint("CENTER", TargetFrameHealthBar, "CENTER", 0, 0)
    TargetFrameTextureFrameName:ClearAllPoints()
    TargetFrameTextureFrameName:SetPoint("BOTTOMRIGHT", TargetFrame, "TOP", 0, -20)
    TargetFrameHealthBar:ClearAllPoints()
    TargetFrameHealthBar:SetPoint("TOPLEFT", 5, -24)
    TargetFrameHealthBar:SetHeight(18)
    TargetFrameTextureFrameHealthBarText:ClearAllPoints()
    TargetFrameTextureFrameHealthBarText:SetPoint("RIGHT", TargetFrameHealthBar, 0, 0)
    TargetFrameManaBar:ClearAllPoints()
    TargetFrameManaBar:SetPoint("TOPLEFT", 5, -43)
    TargetFrameManaBar:SetHeight(10)
    TargetFrameTextureFrameManaBarText:ClearAllPoints()
    TargetFrameTextureFrameManaBarText:SetPoint("RIGHT", TargetFrameManaBar, 0, 0)
    TargetFrameTextureFrameManaBarText:SetFont(TargetFrameTextureFrameManaBarText:GetFont(), 10, "OUTLINE")

    TargetFrame.threatNumericIndicator:SetPoint("BOTTOM", PlayerFrame, "TOP", 75, -22)

 
    ---------------------------------------------------
    -- FOCUS
    --------------------------------------------------- 
    FocusFrame.nameBackground:Hide()
    FocusFrame.deadText:ClearAllPoints()
    FocusFrame.deadText:SetPoint("CENTER", FocusFrameHealthBar, "CENTER", 0, 0)
    FocusFrameTextureFrameName:ClearAllPoints()
    FocusFrameTextureFrameName:SetPoint("BOTTOMRIGHT", FocusFrame, "TOP", 0, -20)
    FocusFrameHealthBar:ClearAllPoints()
    FocusFrameHealthBar:SetPoint("TOPLEFT", 5, -24)
    FocusFrameHealthBar:SetHeight(18)
    FocusFrameTextureFrameHealthBarText:ClearAllPoints()
    FocusFrameTextureFrameHealthBarText:SetPoint("RIGHT", FocusFrameHealthBar, 0, 0)
    FocusFrameManaBar:ClearAllPoints()
    FocusFrameManaBar:SetPoint("TOPLEFT", 5, -43)
    FocusFrameManaBar:SetHeight(10)
    FocusFrameTextureFrameManaBarText:ClearAllPoints()
    FocusFrameTextureFrameManaBarText:SetPoint("RIGHT", FocusFrameManaBar, 0, 0)
    FocusFrame.threatNumericIndicator:SetWidth(0.01)
    FocusFrame.threatNumericIndicator.bg:Hide()
    FocusFrame.threatNumericIndicator.text:Hide()

    function ColorGradient(perc, ...)
        if(perc ~= perc or perc == inf) then perc = 0 end

        if perc >= 1 then
            local r, g, b = select(select('#', ...) - 2, ...)
            return r, g, b
        elseif perc <= 0 then
            local r, g, b = ...
            return r, g, b
        end

        local num = select('#', ...) / 3
        local segment, relperc = math.modf(perc*(num-1))
        local r1, g1, b1, r2, g2, b2 = select((segment*3)+1, ...)

        return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc
    end
    
    ---------------------------------------------------
    -- CLASS COLOR
    ---------------------------------------------------
    local colour = function(bar, unit)
        if unit == bar.unit then
            local t = { r=0, g=1, b=0 }
            
            if (UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit)) then
                t = { r=.6, g=.6, b=.6}
            elseif (not UnitIsConnected(unit)) then
                t = { r=.6, g=.6, b=.6}
            elseif(UnitIsUnit(unit, "pet") and GetPetHappiness()) then
                local happiness = {
                 [1] = {r=1, g=0, b=0}, -- need.... | unhappy
                 [2] = {r=1, g=1, b=0}, -- new..... | content
                 [3] = {r=0, g=1, b=0}, -- colors.. | happy
                }
                t = happiness[GetPetHappiness()] or { r=0, g=1, b=0 }
            elseif UnitIsPlayer(unit) then
                local _, class = UnitClass(unit)
                t = CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[class] or RAID_CLASS_COLORS[class] or  { r=0, g=1, b=0 }
            elseif UnitReaction(unit, "player") then
                t = FACTION_BAR_COLORS and FACTION_BAR_COLORS[UnitReaction(unit, "player")] or { r=0, g=1, b=0 }
            else
                t = {r=0, g=1, b=0}
            end
            
            bar:SetStatusBarColor(t.r, t.g, t.b)
        end
    end
    
    hooksecurefunc("UnitFrameHealthBar_Update", colour)
    hooksecurefunc("HealthBar_OnValueChanged", function(self)
       colour(self, self.unit)
    end)

    ---------------------------------------------------
    -- TARGET AURA STYLE & SET POINT
    ---------------------------------------------------
    local setStyleAura = function(bname, index, isDebuff)
        local button = _G[bname..index]
        local icon   = _G[bname..index.."Icon"]
        local cd     = _G[bname..index.."Cooldown"]
        local count  = _G[bname..index.."Count"]

        if button.bd==nil then
            button.bd = button:CreateTexture(bname..index.."Overlay", "BORDER")
            button.bd:SetAllPoints(button)
            button.bd:SetTexture("Interface\\Buttons\\UI-TotemBar")
            button.bd:SetTexCoord(1 / 128, 35 / 128, 207 / 256, 240 / 256)
            button.bd:SetDesaturated(1)
            button.bd:SetVertexColor(.4, .4, .4)
        end
        
        if icon then
            icon:SetTexCoord(.1, .9, .1, .9)
            icon:SetPoint("TOPLEFT", button, "TOPLEFT", 3, -3)
            icon:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -3, 3)
            if cd then
                cd:SetAllPoints(icon)
            end
        end
        
        if count then
            count:SetDrawLayer("OVERLAY")
            count:ClearAllPoints()
            count:SetFont(count:GetFont(), 10, "OUTLINE")
            count:SetPoint("TOPRIGHT", button, 0, 0)
        end
        
        if isDebuff then
            local dtype = select(5, UnitDebuff("target",index))
            local color
            if (dtype ~= nil) then color = DebuffTypeColor[dtype] else color = DebuffTypeColor["none"] end
            button.bd:SetVertexColor(color.r * .4, color.g * .4, color.b * .4)
        end
    end

    ---------------------------------------------------
    -- TARGET AURA SIZE
    ---------------------------------------------------
    hooksecurefunc("TargetFrame_UpdateAuraPositions", function(self, auraName, numAuras, numOppositeAuras, largeAuraList, updateFunc, maxRowWidth, offsetX)
        local AURA_OFFSET_Y = 1
        local LARGE_AURA_SIZE = config.largeAuraSize
        local SMALL_AURA_SIZE = config.smallAuraSize
        local size
        local offsetY = AURA_OFFSET_Y
        local rowWidth = 0
        local firstBuffOnRow = 1
        for i=1, numAuras do
            if ( largeAuraList[i] ) then
                size = LARGE_AURA_SIZE
                offsetY = AURA_OFFSET_Y + AURA_OFFSET_Y
            else
                size = SMALL_AURA_SIZE
            end
            if ( i == 1 ) then
                rowWidth = size
                self.auraRows = self.auraRows + 1
            else
                rowWidth = rowWidth + size + offsetX
            end
            if ( rowWidth > maxRowWidth ) then
                updateFunc(self, auraName, i, numOppositeAuras, firstBuffOnRow, size, offsetX, offsetY)
                rowWidth = size
                self.auraRows = self.auraRows + 1
                firstBuffOnRow = i
                offsetY = AURA_OFFSET_Y
            else
                updateFunc(self, auraName, i, numOppositeAuras, i - 1, size, offsetX, offsetY)
            end
            setStyleAura(auraName, i, auraName:find("debuff"))
        end
    end)
    

    if SCDB[module.name]["Debuffs On Top"] == true then
        hooksecurefunc("TargetFrame_UpdateDebuffAnchor", function(self, debuffName, index, numBuffs, anchorIndex, size, offsetX, offsetY)
            local buff = _G[debuffName..index]
            buff:ClearAllPoints()
            if ( index == 1 ) then
                buff:SetPoint("TOPLEFT", self, "TOPLEFT", 5, 32)
            elseif ( anchorIndex ~= (index-1) ) then
                buff:SetPoint("BOTTOMLEFT", _G[debuffName..anchorIndex], "TOPLEFT", 0, offsetY)
            else
                buff:SetPoint("TOPLEFT", _G[debuffName..(index-1)], "TOPRIGHT", offsetX, 0)
            end
        end)
    
        hooksecurefunc("TargetFrame_UpdateBuffAnchor", function(self, buffName, index, numDebuffs, anchorIndex, size, offsetX, offsetY)
           local buff = _G[buffName..index]
           if ( index == 1 ) then
              buff:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 5, 32)
           elseif ( anchorIndex ~= (index-1) ) then
              buff:SetPoint("TOPLEFT", _G[buffName..anchorIndex], "BOTTOMLEFT", 0, -offsetY)
           else
              buff:SetPoint("TOPLEFT", _G[buffName..anchorIndex], "TOPRIGHT", offsetX, 0)
           end
        end)
    end
    
    ---------------------------------------------------
    -- SEXY 3D PORTAITS
    ---------------------------------------------------
    function ChangePortaits(self)
        if self.portrait then
            if SCDB[module.name]["3D portaits"] == true then
                if not self.ap then
                    self.ap = _G[self.portrait:GetName().."Model"] or CreateFrame("PlayerModel", self.portrait:GetName().."Model", self.portrait:GetParent())
                    local guid = UnitGUID(self.unit)
                    if (not UnitExists(self.unit) or not UnitIsConnected(self.unit) or not UnitIsVisible(self.unit)) then
                        self.ap:SetModelScale(4.25)
                        self.ap:SetPosition(0, 0, -1.5)
                        self.ap:SetModel("Interface\\Buttons\\talktomequestionmark.mdx")
                    elseif(self.ap.guid ~= guid or (self.unit=="pet" and self.ap.guid == guid)) then
                        self.ap:SetUnit(self.unit)
                        self.ap:SetCamera(0)
                        self.ap.guid = guid
                    end
                    self.ap:SetWidth(self.portrait:GetWidth()*.75)
                    self.ap:SetHeight(self.portrait:GetHeight()*.75)
                    self.ap:SetPoint("CENTER", self.portrait, "CENTER", 0, 0)
                    if self.unit ~= "player" then
                        self.ap:SetFrameLevel(self:GetFrameLevel()-1)
                    else
                        self.ap:SetFrameLevel(self:GetFrameLevel())
                    end
                    self.ap.back = self.ap:CreateTexture(nil, "BACKGROUND")
                    self.ap.back:SetTexture("Interface\\CharacterFrame\\TempPortraitAlphaMaskSmall")
                    self.ap.back:SetAllPoints(self.portrait)
                    self.ap.back:SetVertexColor(0,0,0)
                    self.ap.border = CreateFrame("frame", nil, self)
                    self.ap.border:SetAllPoints(self.portrait)
                    if self.unit ~= "player" then
                        self.ap.border:SetFrameLevel(self:GetFrameLevel())
                    else
                        self.ap.border:SetFrameLevel(self:GetFrameLevel()+1)
                    end
                    self.ap.border.tex = self.ap.border:CreateTexture(nil, "BORDER")
                    self.ap.border.tex:SetTexture("Interface\\CHARACTERFRAME\\TotemBorder")
                    self.ap.border.tex:SetWidth(self.portrait:GetWidth()+16)
                    self.ap.border.tex:SetHeight(self.portrait:GetWidth()+16)
                    self.ap.border.tex:SetPoint("CENTER")
                    self.ap.border.tex:SetVertexColor(0,0,0,.9)
                    
                    self:SetFrameStrata("LOW")
                    self.portrait:SetTexture("")
                else
                    self.portrait:SetTexture("")
                    self.ap:SetCamera(0)
                    local guid = UnitGUID(self.unit)
                    if (not UnitExists(self.unit) or not UnitIsConnected(self.unit) or not UnitIsVisible(self.unit)) then
                        self.ap:SetModelScale(4.25)
                        self.ap:SetPosition(0, 0, -1.5)
                        self.ap:SetModel("Interface\\Buttons\\talktomequestionmark.mdx")
                    elseif (self.ap.guid ~= guid  or (self.unit=="pet" or self.unit=="player" or self.unit:find("party"))) then
                        self.ap:SetUnit(self.unit)
                        self.ap:SetCamera(0)
                        self.ap.guid = guid
                    end
                end
            else
                if (not UnitExists(self.unit) or not UnitIsConnected(self.unit) or not UnitIsVisible(self.unit)) then
                    self.portrait:SetTexture("Interface\\CharacterFrame\\TempPortrait")
                else
                    if self.unit == "player" or self.unit == "pet" then return end
                    local t = CLASS_ICON_TCOORDS[select(2,UnitClass(self.unit))]
                    if t then
                       self.portrait:SetTexture("Interface\\TargetingFrame\\UI-Classes-Circles")
                       self.portrait:SetTexCoord(unpack(t))
                    end
                end
            end
        end
    end
    
    hooksecurefunc("UnitFramePortrait_Update", ChangePortaits)
    
    ---------------------------------------------------
    -- FIX TEXT VALUES FOR HP AND MANA BARS
    ---------------------------------------------------
    local function fixvalue(val)
       if(val >= 1e6) then
          return ('%.2fm'):format(val / 1e6):gsub('%.?0+([km])$', '%1')
       elseif(val >= 1e4) then
          return ('%.1fk'):format(val / 1e3):gsub('%.?0+([km])$', '%1')
       else
          return val
       end
    end

    hooksecurefunc("TextStatusBar_UpdateTextString", function(self)
        if self.TextString and self.currValue and not GetCVarBool("statusTextPercentage") then
            self.TextString:SetText(fixvalue(self.currValue))
        end
    end)

    local uFrame = CreateFrame("frame")
    
    -- setup slash command /tcb
    SLASH_MoveFrames1 = "/mf";
    SlashCmdList["MoveFrames"] = function()
        for i, frame in pairs(allframes) do
            frame.locked = not frame.locked
            if frame.locked then
                frame.drag:EnableMouse(false)
                frame.drag:Hide()
                uFrame:SetScript("OnUpdate", nil)
            else
                frame.drag:EnableMouse(true)
                frame.drag:Show()
                uFrame:SetScript("OnUpdate", MoveAllFrames)
            end
        end
    end
    
end
tinsert(SuperClassic.modules, module) -- finish him!